\ignore{
\begin{code}

{-# OPTIONS --no-positivity-check #-}

open import lib.Prelude 
open List using (_++_) 
open List.In using (_-_)
-- open List.InLists
-- open List.Subsets

open import foc.focbind.univ.examples.DefinedAtoms 
open import foc.focbind.univ.Types 

module foc.focbind.univ.Semantics where

module SignatureInterface where

  record Signature : Set1 where
    field
     InΣ      : Types.Rule -> Set
     -- we'll need a view of the signature as data
     premises : DefinedAtoms.Atom -> List Types.Type
     certify-premises : {D : DefinedAtoms.Atom} {A : Types.Type} -> InΣ(Types._⇐_ D A) -> A ∈ (premises D)

module Semantics(S : SignatureInterface.Signature) where
  open Types public
  open SignatureInterface.Signature S 
\end{code}}

A universe is specified by a inductive datatype of \emph{codes} for
types, along with a function mapping each code to a \ttt{Set}.  In this
case, the \ttt{Type}s above are the codes, and the semantics is
specified in Figure~\ref{fig:sem} by a function \ttt{< Ψ > A}, mapping a
context and a \ttt{Type} to an Agda \ttt{Set}.  The first six cases
interpret the basic types of the simply-typed λ-calculus as their Agda
counterparts, pushing the context inside to the recursive calls.

The next two cases interpret datatypes.  We define an auxiliary datatype
called \ttt{Data} which represents all of the data types defined in the
universe.  \ttt{Data} is indexed by a context and a defined atom, with
the idea that the Agda set \ttt{Data Ψ D} represents the values of
datatype \ttt{D} in context Ψ.  For example, the values of \ttt{Data Ψ
arith} will represent the arithmetic expressions defined by the
signature given in the introduction.  There are two ways to construct a
datatype: (1) apply a datatype constructor to an argument and (2) choose
a variable from Ψ.  Constants are declared in a signature, represented
with a predicate on rules \ttt{InΣ : Rule -> Set}, where \ttt{InΣ R} is
inhabited when the rule \ttt{R} is in the signature.  The first
constructor, written as infix ·, pairs a constant with the
interpretation of the constant's premise.  The second constructor, ▹,
injects a variable from Ψ into \ttt{Data}.\footnote{Agda allows
overloading of datatype constructors between different types, and we
tend to use ▹ for injections from one type to another, as with
\ttt{VarType} above.} See the appendix for the definition of the type
\ttt{∈}, which represents well-scoped de Bruijn
indices~\citep{bellegardehook94subst,birdpaterson99nested,altenkirchreus99gadt}.
A \ttt{DefAtom} \ttt{D} is in the context if there exist credentials
\ttt{c} for which the \ttt{VarType} formed by \ttt{(▹ D \{c\})} is in
the list Ψ.

Finally, we provide a collection of types that deal with the context:
\ttt{Ψ ⇒* A} extends the context (we write \ttt{+} for append); \ttt{□
A} clears the context. The quantifiers \ttt{∀c} and \ttt{∃c} are
interpreted as the corresponding Agda dependent function and pair types.
Finally, \ttt{∀≤ D A} quantifies over contexts \ttt{Ψ'} for which
\ttt{AllEq Ψ' D} holds.  The type \ttt{AllEq} says that every variable
type in \ttt{Ψ} is equal to the given type \ttt{D} (\ttt{List.all} is
true when its argument is true on all elements of the list;
\ttt{eqVarType} is a boolean-valued equality function for variable
types).  (We could internalize \ttt{AllEq Ψ' D} as a type \ttt{alleq D}
in the universe, in which case the bounded quantifier could expressed as
a derived form, but we have not needed \ttt{alleq D} in a positive
position in the examples we have coded so far.)

\begin{figure}
\begin{code}
  AllEq : Vars -> VarType -> Set
  AllEq Ψ D = Check (List.all (eqVarType D) Ψ)

  mutual
   data Data (Ψ : Vars) (D : DefAtom) : Set where
     _·_ : {A : Type} 
         -> InΣ (D ⇐ A)  ->  < Ψ > A  ->  Data Ψ D 
     ▹   : {c : _} -> (▹ D {c}) ∈ Ψ  -> Data Ψ D

   <_>_ : Vars -> Type -> Set 
   ---- basic types
   < Ψ > 1⁺       = Unit
   < Ψ > 0⁺       = Void
   < Ψ > (A ⊗ B)  = (< Ψ > A) × (< Ψ > B)
   < Ψ > (A ⊕ B)  = Either (< Ψ > A) (< Ψ > B)
   < Ψ > (list A) =  List (< Ψ > A)
   < Ψ > (A ⊃ B)  =  (< Ψ > A)  ->  (< Ψ > B)
   ---- data types
   < Ψ > (D⁺ D)   = Data Ψ D
   < Ψ > (D #)    = D ∈ Ψ
   ---- context manipulation
   < Ψ > (Ψnew ⇒* A) = <  Ψ + Ψnew  > A
   < _ > (□ A)        = < [] > A
   < Ψ > (∃c τ)      = Σ \ Ψ' -> < Ψ > (τ Ψ')
   < Ψ > (∀c τ)      = (Ψ' : Vars) -> < Ψ > (τ Ψ')
   < Ψ > (∀≤ D A) = 
     (Ψ' : Vars) -> AllEq Ψ' D -> < Ψ + Ψ' > A
\end{code}
\ignore{
\begin{code}
   -- could be derived forms
   < Ψ > (∃⇒ A) = Σ \ Ψ' -> < Ψ + Ψ' > A
   < Ψ > (∀⇒ A) = (Ψ' : Vars) -> < Ψ + Ψ' > A
   < _ > ([ Ψ ]* A) = < Ψ > A
\end{code}}
\caption{Semantics}
\label{fig:sem}
\end{figure}

Agda does not accept the positivity of \ttt{Data}, and indeed, in our
examples below, we will write signatures for general recursive types.  In
future work, we would like to investigate ways of positivity-checking
the signature \ttt{InΣ} and explaining the construction of the inductive
types in the universe to Agda, e.g. using
containers~\citep{abbot+05constructingspt}.

\ignore{
Because types tend to start with ∀⇒, we define ∀⇛ to avoid the ++ [] that ⊡ (∀⇒ A) would give you
}

We also define versions of □ and ∀⇒ that construct Agda \ttt{Set}s;
these are convenient for typing inhabitants of the of elements of the
universe.

\begin{code}
  ⊡ : Type -> Set
  ⊡ A = < [] > A
  
  ∀⇛_ : Type -> Set
  ∀⇛_ A = (Ψ : Vars) -> < Ψ > A
\end{code}

\ignore{
\begin{code}
  ∀C_ : (Vars -> Type) -> Set
  ∀C_ A = (Ψ : Vars) -> < [] > (A Ψ)

  infix 1 ∀⇛_ -- so they extend as far to the right as possible
  infix 1 ∀C_ -- so they extend as far to the right as possible

  -- these come up every once in a while
  rassoc : (A : Type) (Ψ1 Ψ2 Ψ3 : Vars) -> < (Ψ1 ++ Ψ2) ++ Ψ3 > A -> < Ψ1 ++ (Ψ2 ++ Ψ3) > A 
  rassoc A Ψ1 Ψ2 Ψ3 e = Id.subst (\ Ψ -> < Ψ > A) (List.append-assoc {_} {Ψ1} {Ψ2} {Ψ3} ) e 

  lassoc : (A : Type) (Ψ1 Ψ2 Ψ3 : Vars) -> < Ψ1 ++ (Ψ2 ++ Ψ3) > A -> < (Ψ1 ++ Ψ2) ++ Ψ3 > A 
  lassoc A Ψ1 Ψ2 Ψ3 e = Id.subst (\ Ψ -> < Ψ > A) (Id.sym (List.append-assoc {_} {Ψ1} {Ψ2} {Ψ3} )) e 

  no[] : (A : Type) (Ψ : Vars) -> < Ψ ++ [] > A -> < Ψ > A 
  no[] A Ψ e = Id.subst (\ Ψ -> < Ψ > A) (List.append-rh-[] Ψ) e

  add[] : (A : Type) (Ψ : Vars) -> < Ψ > A  -> < Ψ ++ [] > A 
  add[] A Ψ e = Id.subst (\ Ψ -> < Ψ > A) (Id.sym (List.append-rh-[] Ψ)) e

\end{code}}
