The grammar for the types of our language is as follows:

\[
\begin{array}{lrcl}
\text{Defined atoms} & D & ::= & \ldots \\
\text{Var. Types} & C & ::= & \text{(a subset of $D$)} \\
\text{Contexts} & \Psi & ::= & [] \mid (\Psi , C) \\
\text{Types} & A & ::= & 0⁺ \mid 1⁺ \mid A ⊗ B \mid A ⊕ B \mid \dsd{list} A \mid A ⊃ B \\
& & &  \dsd{D⁺}\, D \mid D\# \mid Ψ ⇒^* A \mid □ A \\
& & &  ∀_{\dsd{c}} \: ψ.A \mid ∃_{\dsd{c}} \: ψ.A \mid ∀_{\mathord{≤} C} A
\end{array}
\]
%

The language is parametrized by a class of defined atoms $D$, which are
the names of datatypes.  A subset of these names are variable types,
which are allowed to appear in contexts.  This distinguishes certain
types $C$ which may be populated by variables from other types $D$ which
may not.  This definition of \ttt{VarType} permits only variables of
base type, rather than the full language of higher-order rules that we
considered in previous work~\citep{lzh08focbind}.  Contexts are lists of
variable types, written with 'cons' on the right.

The types on the first line have their usual meaning.  The type
$\dsd{D⁺}\, D$ is the datatype named by $D$.  Following
Delphin~\citep{poswolsky+08delphin}, we include a type $D\#$ classifying
only the variables of type $D$.  The type $Ψ ⇒^* A$ classifies
inhabitants of $A$ in the current context extended with $Ψ$.  The type
$□ A$ classifies closed inhabitants of $A$.  The types $∀_{\dsd{c}}$ and
$∃_{\dsd{c}}$ classify universal and existential context quantification;
$∀_{\mathord{≤}} C A$ provides bounded context quantification over
contexts containing only the type $C$.

\subsubsection{Agda implementation}

\ignore{
\begin{code}
open import lib.Prelude 
open List using (_++_) 
open List.In using (_-_)
-- open List.InLists
-- open List.Subsets

open import foc.focbind.univ.examples.DefinedAtoms 

module foc.focbind.univ.Types where

module Types where
\end{code}}

We now represent these types in Agda.  Those readers who are not fluent
in dependent programming can find a review of Agda syntax, well-scoped
de Bruijn indices, and universes in Appendix~\ref{sec:bootcamp}.  We
represent defined atoms, variable types and contexts as follows:

\begin{code}
 DefAtom = DefinedAtoms.Atom

 data VarType : Set where
   ▹ : (D : DefAtom) {_ : Check(DefinedAtoms.world D)} 
      -> VarType

 Vars = List VarType 
\end{code} 
% 
\ttt{DefinedAtoms.Atom} is a parameter that we will instantiate later.
\ttt{DefinedAtoms.world} returns true when \ttt{D} is allowed to appear
in the context; \ttt{Check} turns this boolean into a proposition
(\ttt{Check True} is the unit type; \ttt{Check False} is the empty
type).  A \ttt{VarType} is thus a pair of an atom along with the
credentials allowing it to appear in contexts.  
%% We make the credentials an implicit argument (see the appendix for a
%% review of implicit and explicit arguments)---in typical cases, the
%% condition will reduce to \ttt{True}.

\ignore{
\begin{code}
 un▹ : VarType -> DefAtom
 un▹ (▹ D) = D

 eqVarType : VarType -> VarType -> Bool
 eqVarType (▹ D) (▹ D') = DefinedAtoms.eq D D'

 eqVarType-refl : (D : VarType) -> Check(eqVarType D D)
 eqVarType-refl (▹ D) = DefinedAtoms.eq-refl D

 _,,_ : Vars -> VarType -> Vars
 Ψ ,, C = C :: Ψ
 infixl 30 _,,_

 _+_ : Vars -> Vars -> Vars
 Ψ + Ψ' = Ψ' ++ Ψ

 _≤_ : Vars -> Vars -> Set
 _≤_ Ψ Ψ' = ({D : VarType} -> D ∈ Ψ -> D ∈ Ψ')
\end{code}
}

We represent the syntax of types in Agda as follows:

\begin{code}   
 data Type : Set where
   ---- types that have their usual meaning
   1⁺    : Type
   _⊗_   : Type -> Type -> Type
   0⁺    : Type
   _⊕_   : Type -> Type -> Type
   list_ : Type -> Type
   _⊃_   : Type -> Type -> Type
   ---- datatypes and context manipulation
   D⁺    : DefAtom -> Type
   _#    : VarType -> Type
   _⇒*_  : Vars -> Type -> Type
   □     : Type -> Type
   ∀c    : (Vars -> Type) -> Type
   ∃c    : (Vars -> Type) -> Type
   ∀≤    : VarType -> Type -> Type
\end{code}

\ignore{
\begin{code}
   -- compound connectives:
   -- interalized so the tactics for weakening can recognize them:
   ∀⇒_ : Type -> Type
   ∃⇒_ : Type -> Type
   -- eliminate the (++ []) that you get out of treating this as a two-step derived form;
   -- this avoids some propositional equalities
   [_]*_ : Vars -> Type -> Type
\end{code}}

The only subtlety in this definition is that we represent the bodies of
\ttt{∀c} and \ttt{∃c} by \emph{computational functions} in Agda.  This
choice has some trade-offs: on the one hand, it means that the bodies of
quantifiers can be specified by any Agda computation (e.g. by recursion
over the domain).  On the other hand, it makes it difficult to analyze
the syntax of \ttt{Type}s, because there is no way to inspect the body
of the quantifier.  Indeed, this caused problems for our implementation
of the structural properties, which we solved by adding certain
instances of the quantifiers, which would otherwise be derived forms, as
separate \ttt{Type} constructors.  In future work, we may pursue a more
syntactic treatment of the quantifiers (which would of course be easier
if we had good support for variable binding\ldots).

A \emph{rule}, which is the type of a datatype constructor, pairs the
defined atom being constructed with a single premise type (no/multiple
premises can be encoded using 1⁺ and ⊗):

\begin{code}
 data Rule : Set where
   _⇐_ : DefAtom -> Type -> Rule
\end{code}
%
We will make use of a few derived forms:

\begin{itemize}
\item We write (\ttt{∀⇒ A}) for \verb|(∀c \Ψ -> Ψ ⇒* A)|, and similarly for ∃⇒
(note that \verb|\ x -> e| introduces an anonymous function).
This type quantifies over a context Ψ and immediately binds it around A.  
Similarly, we write \ttt{[ Ψ ]* A} for □ (Ψ ⇒* A)

\item We write \ttt{(C ⇒ Ψ)} for \ttt{⇒*} with a single premise.  

\item We write \ttt{(C ⁺)} for \ttt{(D⁺ C)} when \ttt{C} is a variable type.

\item We write \ttt{bool} and \ttt{A option} for the encoding
with 1⁺ and ⊕.

\end{itemize}

\ignore{
\begin{code}
 ax : DefAtom -> Rule
 ax a = a ⇐ 1⁺

 _⇒_ : VarType -> Type -> Type
 D ⇒ A = [ D ] ⇒* A

 _⁺ : (D : VarType) -> Type
 (▹ D) ⁺ = D⁺ D

 bool : Type
 bool = 1⁺ ⊕ 1⁺ 

 _option : Type -> Type
 A option = A ⊕ 1⁺ 

 _extendsfor_ : Vars -> VarType -> Type
 Ψ extendsfor D = (D #) ⊃ [ Ψ ]* (D #)

 var2var : VarType -> Vars -> VarType -> Type
 var2var D Ψ D' = ([ Ψ ]* (D #)) ⊃ (D' #)
\end{code}}

\ignore{
\begin{code}
 -- application seems to be 20
 infix  99 _⁺  -- should bind as tightly as possible
 infix  99 _#
 infix  99 D⁺  
 infixl 10 _⇐_ -- looser than connectives
 infixr 15 _⊃_ -- implication binds looser than prods and sums
 infixr 16 _⊕_ -- products tighter than sums
 infixr 17 _⊗_
 infixr 15 _⇒*_ -- looks funny whenever you don't use it at the same prec as implication
 infixr 15 _⇒_  
 infixr 19 [_]*_  -- tighter than everyone
 infix  1 ∀⇒_ -- so they extend as far to the right as possible
 infix  1 ∃⇒_
 infix  1 ∀≤

{-
  VarRel = Type -> Vars -> Vars -> Set

  record MapDefCases : Set where
   field
    defAtom : DefAtom -> Type
    defVar  : VarType -> Type

  mapDefs : MapDefCases -> Type -> Type
  mapDefs c 1⁺ = 1⁺
  mapDefs c 0⁺ = 0⁺
  mapDefs c (A ⊗ B) = mapDefs c A ⊗ mapDefs c B
  mapDefs c (A ⊕ B) = mapDefs c A ⊕ mapDefs c B
  mapDefs c (A ⊃ B) = mapDefs c A ⊃ mapDefs c B
  mapDefs c (□ A) = □ (mapDefs c A)
  mapDefs c (list A) = list (mapDefs c A)
  mapDefs c (D⁺ D) = MapDefCases.defAtom c D
  mapDefs c (D #) = MapDefCases.defVar c D
  mapDefs c (Ψ ⇒* A) = Ψ ⇒* (mapDefs c A)
  mapDefs c (∃c τ) = ∃c (\ Ψ -> mapDefs c (τ Ψ))
  mapDefs c (∀c τ) = ∀c (\ Ψ -> mapDefs c (τ Ψ))
  mapDefs c (∀⇒ A) = ∀⇒ (mapDefs c A)
  mapDefs c (∃⇒ A) = ∃⇒ (mapDefs c A)
-}

\end{code}
}