\ignore{
\begin{code}
open import lib.Prelude 
open List using (_++_) 
open List.In using (_-_)
-- open List.InLists
-- open List.Subsets

open import foc.focbind.univ.examples.DefinedAtoms 
open import foc.focbind.univ.Types 
open import foc.focbind.univ.Semantics 
open import foc.focbind.univ.Structural
open import foc.focbind.univ.examples.Sig

module foc.focbind.univ.examples.Closure where
  open Sig
  open Semantics(theSig)
  open Structural(theSig)
\end{code}}

Next, we implement a closure-based evaluator for the untyped λ-calculus.
λ-terms and closures are represented by types \ttt{exp} and \ttt{clos}
as follows:

\noagda\begin{code}
  lam  : InΣ (expA ⇐ (expC ⇒ exp))
  app  : InΣ (expA ⇐ exp ⊗ exp)

  closure : InΣ (closA ⇐ (∃⇒ (expC ⇒ exp) ⊗ 
                                (expC #  ⊃  □ clos)))
\end{code}
%

Expressions are defined by the usual signature, as in LF.  The type of
closures, \ttt{clos}, is a recursive type with one constructor
\ttt{closure}.  The premise of \ttt{closure} should be read as follows:
a closure is constructed from a triple \ttt{(Ψ , e , σ)}, where (1) Ψ is
an existentially quantified context; (2) \ttt{e} is an expression in Ψ
with an extra free variable, which represents the body of a
λ-abstraction; and (3) σ is a substitution of of closed closures for all
the variables in Ψ.  We represent a substitution as a function that maps
each expression variable in the context (classified by the type
\ttt{expC \#}) to a closure.  The type of the premise provides a
succinct description of all of this: ∃⇒ introduces the variables in the
existentially quantified context into scope without explicitly naming
the context; ⇒ extends the context with an additional variable; (expC
\#) ranges over all of the variables in scope. For
comparison, the meaning of this type in Ψ is: 
\noagda\begin{code}
Σ \(Ψ' : Vars) -> (Data (Ψ + Ψ' ,, expC) expA) × 
                   (expC ∈ (Ψ + Ψ')  ->  Data [] closA)
\end{code}
(where we write \ttt{,,} for cons on the right).  

Evaluation maps an expression to a closed closure (\ttt{□ clos}), given
a substitution of closed closures for each expression variable in the
context:

\begin{code}
  env : Type
  env = expC #  ⊃  □ clos

  eval : ⊡ (∀⇒ exp ⊃ env ⊃ □ clos)
  eval Ψ (▹ x) σ =  σ x
  eval Ψ (lam · e) σ =  closure · (Ψ , e , σ) 
  eval Ψ (app · (e1 , e2)) σ 
   with eval Ψ e1 σ                
  ... | closure · (Ψ' , e' , σ') = 
    eval (Ψ' ,, expC) e' 
         (extend{(□ clos)} _ σ' (eval Ψ e2 σ)) 
  ... | ▹ x = impossible x
\end{code}
%
A variable is evaluated by applying the substitution. A \ttt{lam}
evaluates to the obvious closure.  
To evaluate an application, we first evaluate the function position.  
To a first approximation, the reader may think of Agda's \ttt{with}
syntax as a \ttt{case} statement in the
body of the clause, with each branch marked by \ttt{... |}.  
Case-analyzing the evaluation of \ttt{e1} gives two cases: (1) the value
is constructed by the constructor \ttt{closure}; (2) the value is a
variable.  

In the first, case we evaluate the body of the closure in an extended
environment.  The call to the function \ttt{extend} extends the
environment σ' so that the last variable is mapped to the value of
\ttt{e2}. The definition of \ttt{extend} is as follows:

\noagda\begin{code}
  extend : {A : Type} {D : VarType} 
         -> (∀⇛ (D # ⊃ A) ⊃ A ⊃ (D ⇒ D #) ⊃ A)
  extend Ψ σ new i0 = new
  extend Ψ σ new (iS i) = σ i
\end{code}
At the call site of extend, we must explicitly supply the type \ttt{A}
(in this case \ttt{□ clos}) to help out type reconstruction.  The
underscore stands for the instantiation of the ∀⇛, which is marked as an
explicit argument, but can in this case be inferred.  

The second case is contradicted using the function \ttt{impossible},
which refutes the existence of a variable at a non-\ttt{VarType}---which
\ttt{clos} is, because we never wish to have \ttt{clos} variables.

The context argument Ψ to \ttt{eval} does not play an interesting role
in the code, but Agda's type reconstruction requires us to supply it
explicitly at each recursive call.  In future work, we may consider
whether this argument can be inferred.  Agda is unable to verify the
termination of this \ttt{eval}uator for the untyped λ-calculus, as one
would hope.

When writing this code, one mistake a programmer might make is to
evaluate the body of the closure in σ instead of σ', which would give
dynamic scope.  If we make this mistake, Agda highlights the occurrence
of σ and helpfully reports the type error that \verb|Ψ' != Ψ|,
indicating that the context of the expression does not match the context
of the substitution.

%% In this example, we always use the type \ttt{clos} directly under a box,
%% as in \ttt{□ clos}.  It is possible to change the signature so that the
%% \ttt{□} is moved into the premise of \ttt{clos}, and change the type of
%% \ttt{eval} to \ttt{(∀⇒ exp ⊃ env ⊃ clos)}.  However, in this version, we
%% must explicitly coerce the result of the application case from \ttt{(Ψ'
%% ,, expC)} back to Ψ---which we can do by simply unpacking the closure's
%%  triple and then repacking it.  The version above avoids this detail.

\paragraph{Comparison.}  

It is unclear to us how one would express this example in Delphin or
Beluga, as Delphin lacks existential context quantification and □, and
Beluga lacks the parameter type \ttt{exp \#}, so our definition of
\ttt{clos} cannot be straightforwardly ported to either of these
languages.  Beluga provides a built-in type of substitutions, written
[Ψ']Ψ, so one might hope to represent closures using the type
$∃ψ. ([ψ,x:\ttt{exp}]\ttt{exp}) × [.]ψ$.  However, the second component
of this pair associates an \emph{expression} with each expression
variable in ψ, whereas we want to associate a \emph{closure} with each
expression variable in ψ in this example.  One could presumably port
this code to FreshML~\citep{shinwell+03freshmlicfp}, but the type system
would not enforce the invariant that closures are in fact closed.  To
our knowledge, a proof of this property for this example has not been
attempted in Pure FreshML~\citep{pottier07staticfreshml}, though we know
of no reason why it would not be possible.

\ignore{
  -- fiddly detail: ⊡ ∀⇒ works better than ∀⇛ here,
  -- because having the extra ++ [] interacts well with the □ clos
}

