\ignore{
\begin{code}
{-# OPTIONS --no-termination-check #-}

open import lib.Prelude 
open List using (_++_) 
open List.In using (_-_)
-- open List.InLists
-- open List.Subsets

open import foc.focbind.univ.examples.DefinedAtoms 
open import foc.focbind.univ.Types 
open import foc.focbind.univ.Semantics 
open import foc.focbind.univ.Structural
open import foc.focbind.univ.examples.Sig

module foc.focbind.univ.examples.VariableManip where
  open Sig
  open Semantics(theSig)
  open Structural(theSig)
\end{code}}

Next, we consider a suite of simple variable manipulations.

\subsubsection{Size}

First, we compute the size of a λ-term.  Addition is defined as usual,
with a contradictory variable case because no \ttt{natA} variables are
allowed.

\begin{code}
  plus : ⊡ (nat ⊃ nat ⊃ nat)
  plus (zero · _) m = m
  plus (succ · n) m = succ · (plus n m)
  plus (▹ x)      m = impossible x

  size : (∀⇛  exp ⊃ □ nat)
  size Ψ (▹ x)             = succ · (zero · _)
  size Ψ (app · (e1 , e2)) = succ · (plus (size Ψ e1) 
                                          (size Ψ e2))
  size Ψ (lam · e)         = succ · (size (Ψ ,, expC) e)
\end{code}
%
Agda successfully termination-checks these functions.  

The type of size expresses that it returns a closed natural number.  For
comparison, we implement a second version that does not make this
invariant explicit:

\begin{code}
  size' : ⊡ (∀≤ expC (exp ⊃ nat))
  size' Ψ bound (▹ x) = succ · (zero · _)
  size' Ψ bound (app · (e1 , e2)) = 
   succ · (plus' Ψ bound (size' Ψ bound e1) 
                         (size' Ψ bound e2))  where
    plus' : ⊡ (∀≤ expC (nat ⊃ nat ⊃ nat))
    plus' Ψ b = weaken*/bounded (nat ⊃ nat ⊃ nat) Ψ b [] 
                                plus  
  size' Ψ bound (lam · e)         = 
    strengthen nat _ (size' (Ψ ,, expC) bound e)
\end{code}
%
Without the □, size must return a number in context Ψ: in the
application case, we must weaken \ttt{plus} into Ψ, and in the \ttt{lam}
case we must strengthen the extra \ttt{expC} variable out of the
recursive call.  Strengthening expression variables out of natural
numbers is permitted by our implementation of the structural properties
because natural numbers cannot mention expressions; we use a
subordination-like analysis to determine this~\citep{virga99thesis}.  To
do ensure that these weakenings and strengthenings are permitted, we
type \ttt{size} with a bounded quantifier over \ttt{exp} variables to
ensure that these

\textbf{Comparison.}  The first version is similar to what one writes in
FreshML, except in that setting there is no need to pass around a
context Ψ.  In the second version, the strengthening of the recursive
result in the \ttt{lam} case is analogous to the need, in FreshML
2000~\citep{pitts00freshml2000}, to observe that \ttt{nat} is pure
(always has empty support); FreshML~\citep{shinwell+03freshmlicfp} does
not require this.

In Beluga, one can express either the first or second
versions.  In Twelf and Delphin, one can only express the second
variation, as these languages do not provide □.  However, the
Twelf/Delphin/Beluga syntax for weakening and strengthening is terser
than what we have been able to construct in Agda: weakening is handled
by world subsumption and is not marked in the proof term; strengthening
is marked by pattern-matching the result of the recursive call and
marking those variables that \emph{do} occur, which in this case does
not include the expression variable.  For example, the \ttt{lam} case of
size in Twelf looks like this:

\noagda\begin{code}
- : size (lam ([x] E x)) (succ N)
      <- ({x : exp} size (E x) N).  
\end{code}  
%
Twelf's coverage checker verifies that expression variables can be
strengthened out of natural numbers when checking this case.  
We would like to explore a similarly terse syntax for
weakening/strengthening in future work.

\subsubsection{Counting occurrences of a variable}

A simple variation is to count the number of occurrences of a
distinguished free variable.  The input to this function has type
\ttt{(expC ⇒ exp)}, and we count the occurrences of the bound variable:

\begin{code}
  cnt : ∀⇛ (expC ⇒ exp)  ⊃  □ nat
  cnt Ψ (▹ i0) = succ · (zero · _)
  cnt Ψ (▹ (iS _)) =  zero · _ 
  cnt Ψ (app · (e1 , e2)) = plus (cnt Ψ e1) (cnt Ψ e2)   
  cnt Ψ (lam · e) = cnt (Ψ ,, expC) (exchange2 exp Ψ e) 
\end{code}
%
In the first two cases, we pattern-match on the variable: when it is the
last variable, the last variable occurs once; when it is not, it occurs
zero times.  The \ttt{lam} case recurs on the exchange of \ttt{e}, so
that the last variable remains the one we are looking for.
Agda fails to termination-check this example because it recurs on the
result of exchange.  Because this use of exchange is a common recursion
pattern for \ttt{(exp -> exp)} in Twelf, we plan to consider a derived
induction principle that covers this case in future work.

\paragraph{Comparison.} Pattern-matching on variables is represented
using higher-order metavariables in Twelf/Delphin/Beluga and using
equality tests on names in FreshML.  The exchange needed in the
\ttt{lam} case is written as a substitution in the Twelf/Delphin/Beluga
version of this clause.  In Twelf one would write:

\noagda \begin{code}
  - : cnt ([x] lam ([y] E x y)) N
       <- ({y:exp} cnt ([x] E x y) N).
\end{code}
%  
In the input to this clause, the metavariable \ttt{E}, which stands for
the body of the function, refers to the last variable in the context
(the \ttt{lam}-bound variable) as \ttt{y} and the second-last variable
(the variable being counted) as \ttt{x}.  In the recursive call, \ttt{y}
is exchanged past the binding of \ttt{x}, so the instantiation \ttt{E x
y} swaps ``last'' and ``second-last''.  

\subsubsection{Computing free variables}  

Next, we consider a function computing the free variables of an
expression, of type \ttt{(∀⇛ exp ⊃ list (expC \#))}---in any context,
this function accepts an expression in that context and produces a list
of variables in that context.  This typing ensures that we do not
accidentally return a bound variable.

\begin{code}
  remove : {D : VarType} 
         -> (∀⇛  (D ⇒ list (D #))  ⊃  list (D #))
  remove Ψ []             = [] 
  remove Ψ (i0 :: ns)     =  (remove Ψ ns) 
  remove Ψ ((iS i) :: ns) = i :: (remove Ψ ns)


  fvs : (∀⇛  exp  ⊃  list (expC #))
  fvs Ψ (▹ x) = [ x ] 
  fvs Ψ (lam · e) = remove Ψ (fvs (Ψ ,, expC) e) 
  fvs Ψ (app · (e1 , e2)) = (fvs Ψ e1) ++ (fvs Ψ e2)
\end{code}
%% 
In the \ttt{lam} case, we use the helper function \ttt{remove} to remove
the \ttt{lam}-bound variable from the recursive result.  The function
\ttt{remove} takes a list of variables, itself with a distinguished free
variable, and produces a list of variables without the distinguished
variable.  If the programmer were to make a mistake in the second clause
by accidentally including \ttt{i0} in the result, he would get a type
error.  Agda successfully termination-checks this example.

\textbf{Comparison.}  
For comparison with FreshML~\citep{shinwell+03freshmlicfp}, the type
given to \ttt{remove} here is analogous to their Figure~6:
\noagda\begin{code}
remove : (<name> (name list)) -> name list
\end{code}
where <a>τ is a nominal abstractor.  
The authors comment that they prefer the version of \ttt{remove} in
their Figure 5:
\noagda\begin{code}
remove : name -> (name list) -> name list
\end{code}
where the name to removed is specified by the first argument,
rather than using a binder.

Using dependent types, we can type this second version of remove as
follows:
\noagda\begin{code}
remove : (Ψ : Vars) (i : exp ∈ Ψ) 
       -> List (exp ∈ Ψ) -> List (exp ∈ (Ψ - i))
\end{code}
where \ttt{Ψ - i} removes the indicated element element from the list.  
This type is of course expressible in Agda, but we have not yet
integrated dependent types into our universe.

\subsubsection{η-Contraction}  

In Twelf/Delphin/Beluga, one can recognize η-redices by writing a
meta-variable that is not applied to all enclosing locally bound
variables.  E.g. in Twelf one would write

\noagda\begin{code}
- : contract (lam [x] app F x) F.
\end{code}
%
The metavariable \ttt{F:exp} is bound outside the scope of \ttt{x}, and
thus stands only for terms that do not mention \ttt{x}.  (To allow it to
mention \ttt{x}, we would bind \ttt{F:exp -> exp} and write \ttt{(F x)} in
place of \ttt{F}.)

Unfortunately, Agda does not provide this sort of pattern matching for
our encoding---pattern variables are always in the scope of all
enclosing local binders---so we must explicitly call a strengthening
function that checks whether the variable occurs:  

\begin{code}
  strengthen? : ∀⇛ (expC ⇒ exp) ⊃ exp option
  strengthen? Ψ (▹ i0)     = Inr _
  strengthen? Ψ (▹ (iS i)) = Inl (▹ i)
  strengthen? Ψ (app · (e1 , e2)) 
   with strengthen? Ψ e1 | strengthen? Ψ e2
  ... | Inl e1'          | Inl e2' = Inl (app · (e1' , e2'))
  ... | _                | _       = Inr _
  strengthen? Ψ (lam · e) 
   with strengthen? (Ψ ,, expC) (exchange2 exp Ψ e)
  ... | Inl e' = Inl (lam · e')
  ... | _      = Inr _

  contract-η : ∀⇛ exp ⊃ exp option
  contract-η Ψ (lam · (app · (f , ▹ i0))) = strengthen? Ψ f 
  contract-η Ψ _ = Inr <>
\end{code}

We conjecture that \ttt{strengthen?} could be implemented
datatype-generically for all purely positive types (no ⊃ or ∀c or ∀≤).
However, it is not possible to decide whether a variable occurs in the
values of these computational types (cf. FreshML, where it is not
possible to decide whether a name is in the support of a function).

This strengthening function is not an instance of the generic \ttt{map}
that we define below, as it changes the type of the term (\ttt{exp} to
\ttt{exp option}); in future work, we plan to consider a more general
traversal that admits this operation.

\ignore{
\begin{code}
  module Test where

    -- cnt = 0
    e1 : ⊡ (expC ⇒ exp)
    e1 = lam · (lam · (app · (▹ i0 , ▹ (iS i0))))

    -- cnt = 1
    e2 : ⊡ (expC ⇒ exp)
    e2 = lam · (lam · (app · (▹ i0 , ▹ (iS (iS i0)))))

    -- cnt = 2
    e3 : ⊡ (expC ⇒ exp)
    e3 = lam · (lam · (app · (▹ (iS (iS i0)) , ▹ (iS (iS i0)))))
\end{code}}
